Package: bayesImageS
Type: Package
Title: Bayesian Methods for Image Segmentation using a Potts Model
Version: 0.7-0
Date: 2025-10-10
Encoding: UTF-8
Authors@R: c(
    person("Matt", "Moores", role = c("aut", "cre", "cph"), email = "mmoores@gmail.com", comment = c(ORCID = "0000-0003-4531-3572")),
    person("Dai", "Feng", role="ctb"),
    person("Kerrie", "Mengersen", role=c("aut", "ths"), email="k.mengersen@qut.edu.au", comment = c(ORCID = "0000-0001-8625-9168")))
Description: Various algorithms for segmentation of 2D and 3D images, such
    as computed tomography and satellite remote sensing. This package implements
    Bayesian image analysis using the hidden Potts model with external field
    prior of Moores et al. (2015) <doi:10.1016/j.csda.2014.12.001>.
    Latent labels are sampled using chequerboard updating or Swendsen-Wang.
    Algorithms for the smoothing parameter include pseudolikelihood, path sampling,
    the exchange algorithm, approximate Bayesian computation (ABC-MCMC and ABC-SMC),
    and the parametric functional approximate Bayesian (PFAB) algorithm. Refer to
    Moores, Pettitt & Mengersen (2020) <doi:10.1007/978-3-030-42553-1_6> for an overview
    and also to <doi:10.1007/s11222-014-9525-6> and <doi:10.1214/18-BA1130> for
    further details of specific algorithms.
License: GPL (>= 2) | file LICENSE
URL: https://bitbucket.org/Azeari/bayesimages,
        https://mooresm.github.io/bayesImageS/
BugReports: https://bitbucket.org/Azeari/bayesimages/jira
LazyData: true
Depends: R (>= 3.5.0)
Imports: Rcpp (>= 1.0.12)
LinkingTo: Rcpp, RcppArmadillo
Suggests: mcmcse, coda, PottsUtils, rstan, knitr, rmarkdown, lattice
VignetteBuilder: knitr
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2025-10-10 02:06:16 UTC; mmoores
Author: Matt Moores [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-4531-3572>),
  Dai Feng [ctb],
  Kerrie Mengersen [aut, ths] (ORCID:
    <https://orcid.org/0000-0001-8625-9168>)
Maintainer: Matt Moores <mmoores@gmail.com>
Repository: CRAN
Date/Publication: 2025-10-10 08:30:11 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-25 10:01:26 UTC; unix
Archs: bayesImageS.so.dSYM
