# Generated by roxygen2: do not edit by hand

export(exactPotts)
export(getBlocks)
export(getEdges)
export(getNeighbors)
export(gibbsGMM)
export(gibbsNorm)
export(gibbsPotts)
export(initSedki)
export(mcmcPotts)
export(mcmcPottsNoData)
export(smcPotts)
export(sufficientStat)
export(swNoData)
export(testResample)
importFrom(Rcpp,sourceCpp)
useDynLib(bayesImageS)
